
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <time.h>

int count_bits(unsigned long n)
{
    unsigned int c;             // c accumulates the total bits set in v
    for (c = 0; n; c++)
        n &= n - 1;             // clear the least significant bit set
    return c;
}

int count_bits2(unsigned long n)
{
    unsigned int c;             // c accumulates the total bits set in v
    for (c = 0; n; n >>= 1)
        if (n & 1)
            ++c;
    return c;
}

int count_bits3(unsigned long n)
{
    unsigned long r = n;

    // Source: bithacks
    // Count set bits in parallel.
    // r = (r & 0x5555555555555555UL) + ((r >> 1) & 0x5555555555555555UL);
    r = r - ((r >> 1) & ~0UL / 3);
    // r = (r & 0x3333...) + ((r >> 2) & 0x3333...);
    r = (r & ~0UL / 5) + ((r >> 2) & ~0UL / 5);
    // r = (r & 0x0f0f...) + ((r >> 4) & 0x0f0f...);
    r = (r + (r >> 4)) & ~0UL / 17;
    // r = r % 255;
    r = (r * (~0UL / 255)) >> ((sizeof(r) - 1) * CHAR_BIT);

    return r;
}

#define N 200000000L
int main()
{
    unsigned long *p = malloc(N * sizeof(unsigned long));
    if (!p)
        return 1;
    for (int i = 0; i < N; ++i)
        p[i] = i;
    clock_t start = clock();
    unsigned long x = 0;
    for (int i = 0; i < N; ++i)
        x += count_bits(p[i]);
    double elapsedTime = (double) (clock() - start) / CLOCKS_PER_SEC;
    printf("time1=%.2f\t", elapsedTime);
    printf("count=%ld\n", x);

    start = clock();
    x = 0;
    for (int i = 0; i < N; ++i)
        x += count_bits2(p[i]);
    elapsedTime = (double) (clock() - start) / CLOCKS_PER_SEC;
    printf("time2=%.2f\t", elapsedTime);
    printf("count=%ld\n", x);

    start = clock();
    x = 0;
    for (int i = 0; i < N; ++i)
        x += count_bits3(p[i]);
    elapsedTime = (double) (clock() - start) / CLOCKS_PER_SEC;
    printf("time3=%.2f\t", elapsedTime);
    printf("count=%ld\n", x);

    free(p);
    return 0;
}
